<?php

namespace Drupal\entity_gallery_slideshow\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for the slideshow.
 *
 * @package Drupal\entity_gallery_slideshow\Controller
 */
class SlideshowController extends ControllerBase {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): SlideshowController {
    $instance = parent::create($container);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    return $instance;
  }

  /**
   * Return the entities slideshow.
   *
   * @param string $entity_type
   *   The entity type.
   * @param string $entity_id
   *   The entity ID.
   * @param string $field_name
   *   The entity field containing the entities.
   * @param string $slide_id
   *   The clicked slide ID.
   * @param string $view_mode
   *   The view mode to render the entity with.
   * @param string $pager
   *   Whether the pager is enabled.
   * @param string $modal_title
   *   The modal title.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   Returns render array.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getSlideshow(
    string $entity_type,
    string $entity_id,
    string $field_name,
    string $slide_id,
    string $view_mode,
    string $pager,
    string $modal_title
  ): AjaxResponse {
    $build = [];

    // Add wrapper to be able to target it with JS.
    $build['wrapper'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['entity-gallery-slideshow'],
        'data-slide' => $slide_id,
      ],
    ];

    // Add rendered entities.
    $parent_entity = $this->entityTypeManager->getStorage($entity_type)
      ->load($entity_id);
    $items = [];

    if ($parent_entity instanceof EntityInterface && $parent_entity->hasField($field_name)) {
      $entities = $parent_entity->{$field_name}->referencedEntities();
      foreach ($entities as $key => $entity) {
        $render = $this->entityTypeManager()
          ->getViewBuilder($entity->getEntityTypeId())
          ->view($entity, $view_mode);
        $render['#slide_id'] = $key;
        $items[] = $render;
      }
    }

    $build['wrapper']['content'] = [
      'items' => $items,
      '#attached' => [
        'library' => [
          'entity_gallery_slideshow/slideshow',
        ],
        'drupalSettings' => [
          'egs' => [
            'pager' => $pager,
          ],
        ],
      ],
      '#cache' => [
        'max-age' => 0,
      ],
    ];

    // Send response.
    $response = new AjaxResponse();
    $dialog_options = [
      'width' => '100%',
      'title' => $modal_title,
      'closeText' => $this->t('Close slideshow'),
      'modal' => TRUE,
      'resizable' => FALSE,
    ];
    $response->addCommand(new OpenModalDialogCommand($this->t('Slideshow'), $build, $dialog_options));

    return $response;
  }

}
